@@ -9,7 +9,9 @@ package mongo
 import (
 	"context"
 	"errors"
+	"fmt"
 	"math"
+	"os"
 	"strconv"
 	"strings"
 	"testing"
@@ -17,8 +19,8 @@ import (
 
 	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/event"
-	"go.mongodb.org/mongo-driver/internal/testutil"
-	"go.mongodb.org/mongo-driver/internal/testutil/assert"
+	"go.mongodb.org/mongo-driver/internal/assert"
+	"go.mongodb.org/mongo-driver/internal/integtest"
 	"go.mongodb.org/mongo-driver/mongo/description"
 	"go.mongodb.org/mongo-driver/mongo/options"
 	"go.mongodb.org/mongo-driver/mongo/readpref"
@@ -32,19 +34,14 @@ var (
 	errorInterrupted int32 = 11601
 )
 
-type wrappedError struct {
-	err error
-}
-
-func (we wrappedError) Error() string {
-	return we.err.Error()
-}
-
-func (we wrappedError) Unwrap() error {
-	return we.err
-}
-
 func TestConvenientTransactions(t *testing.T) {
+	if testing.Short() {
+		t.Skip("skipping integration test in short mode")
+	}
+	if os.Getenv("DOCKER_RUNNING") != "" {
+		t.Skip("skipping test in docker environment")
+	}
+
 	client := setupConvenientTransactions(t)
 	db := client.Database("TestConvenientTransactions")
 	dbAdmin := client.Database("admin")
@@ -79,7 +76,7 @@ func TestConvenientTransactions(t *testing.T) {
 		defer sess.EndSession(context.Background())
 
 		testErr := errors.New("test error")
-		_, err = sess.WithTransaction(context.Background(), func(sessCtx SessionContext) (interface{}, error) {
+		_, err = sess.WithTransaction(context.Background(), func(SessionContext) (interface{}, error) {
 			return nil, testErr
 		})
 		assert.Equal(t, testErr, err, "expected error %v, got %v", testErr, err)
@@ -93,7 +90,7 @@ func TestConvenientTransactions(t *testing.T) {
 		assert.Nil(t, err, "StartSession error: %v", err)
 		defer sess.EndSession(context.Background())
 
-		res, err := sess.WithTransaction(context.Background(), func(sessCtx SessionContext) (interface{}, error) {
+		res, err := sess.WithTransaction(context.Background(), func(SessionContext) (interface{}, error) {
 			return false, nil
 		})
 		assert.Nil(t, err, "WithTransaction error: %v", err)
@@ -113,7 +110,7 @@ func TestConvenientTransactions(t *testing.T) {
 			assert.Nil(t, err, "StartSession error: %v", err)
 			defer sess.EndSession(context.Background())
 
-			_, err = sess.WithTransaction(context.Background(), func(sessCtx SessionContext) (interface{}, error) {
+			_, err = sess.WithTransaction(context.Background(), func(SessionContext) (interface{}, error) {
 				return nil, CommandError{Name: "test Error", Labels: []string{driver.TransientTransactionError}}
 			})
 			assert.NotNil(t, err, "expected WithTransaction error, got nil")
@@ -145,8 +142,8 @@ func TestConvenientTransactions(t *testing.T) {
 			assert.Nil(t, err, "StartSession error: %v", err)
 			defer sess.EndSession(context.Background())
 
-			_, err = sess.WithTransaction(context.Background(), func(sessCtx SessionContext) (interface{}, error) {
-				_, err := coll.InsertOne(sessCtx, bson.D{{"x", 1}})
+			_, err = sess.WithTransaction(context.Background(), func(ctx SessionContext) (interface{}, error) {
+				_, err := coll.InsertOne(ctx, bson.D{{"x", 1}})
 				return nil, err
 			})
 			assert.NotNil(t, err, "expected WithTransaction error, got nil")
@@ -178,8 +175,8 @@ func TestConvenientTransactions(t *testing.T) {
 			assert.Nil(t, err, "StartSession error: %v", err)
 			defer sess.EndSession(context.Background())
 
-			_, err = sess.WithTransaction(context.Background(), func(sessCtx SessionContext) (interface{}, error) {
-				_, err := coll.InsertOne(sessCtx, bson.D{{"x", 1}})
+			_, err = sess.WithTransaction(context.Background(), func(ctx SessionContext) (interface{}, error) {
+				_, err := coll.InsertOne(ctx, bson.D{{"x", 1}})
 				return nil, err
 			})
 			assert.NotNil(t, err, "expected WithTransaction error, got nil")
@@ -344,9 +341,36 @@ func TestConvenientTransactions(t *testing.T) {
 			return nil
 		})
 	})
-	t.Run("commitTransaction timeout does not retry", func(t *testing.T) {
+	t.Run("context error before commitTransaction does not retry and aborts", func(t *testing.T) {
 		withTransactionTimeout = 2 * time.Second
 
+		// Create a special CommandMonitor that only records information about abortTransaction events.
+		var abortStarted []*event.CommandStartedEvent
+		var abortSucceeded []*event.CommandSucceededEvent
+		var abortFailed []*event.CommandFailedEvent
+		monitor := &event.CommandMonitor{
+			Started: func(ctx context.Context, evt *event.CommandStartedEvent) {
+				if evt.CommandName == "abortTransaction" {
+					abortStarted = append(abortStarted, evt)
+				}
+			},
+			Succeeded: func(_ context.Context, evt *event.CommandSucceededEvent) {
+				if evt.CommandName == "abortTransaction" {
+					abortSucceeded = append(abortSucceeded, evt)
+				}
+			},
+			Failed: func(_ context.Context, evt *event.CommandFailedEvent) {
+				if evt.CommandName == "abortTransaction" {
+					abortFailed = append(abortFailed, evt)
+				}
+			},
+		}
+
+		// Set up a new Client using the command monitor defined above get a handle to a collection. The collection
+		// needs to be explicitly created on the server because implicit collection creation is not allowed in
+		// transactions for server versions <= 4.2.
+		client := setupConvenientTransactions(t, options.Client().SetMonitor(monitor))
+		db := client.Database("foo")
 		coll := db.Collection("test")
 		// Explicitly create the collection on server because implicit collection creation is not allowed in
 		// transactions for server versions <= 4.2.
@@ -373,14 +397,13 @@ func TestConvenientTransactions(t *testing.T) {
 			}
 		}()
 
-		// Create context to manually cancel in callback.
-		cancelCtx, cancel := context.WithCancel(bgCtx)
-		defer cancel()
+		// Insert a document within a session and manually cancel context before
+		// "commitTransaction" can be sent.
+		callback := func(ctx context.Context) {
+			transactionCtx, cancel := context.WithCancel(ctx)
 
-		// Insert a document within a session and manually cancel context.
-		callback := func() {
-			_, _ = sess.WithTransaction(cancelCtx, func(sessCtx SessionContext) (interface{}, error) {
-				_, err := coll.InsertOne(sessCtx, bson.M{"x": 1})
+			_, _ = sess.WithTransaction(transactionCtx, func(ctx SessionContext) (interface{}, error) {
+				_, err := coll.InsertOne(ctx, bson.M{"x": 1})
 				assert.Nil(t, err, "InsertOne error: %v", err)
 				cancel()
 				return nil, nil
@@ -389,6 +412,12 @@ func TestConvenientTransactions(t *testing.T) {
 
 		// Assert that transaction is canceled within 500ms and not 2 seconds.
 		assert.Soon(t, callback, 500*time.Millisecond)
+
+		// Assert that AbortTransaction was started once and succeeded.
+		assert.Equal(t, 1, len(abortStarted), "expected 1 abortTransaction started event, got %d", len(abortStarted))
+		assert.Equal(t, 1, len(abortSucceeded), "expected 1 abortTransaction succeeded event, got %d",
+			len(abortSucceeded))
+		assert.Equal(t, 0, len(abortFailed), "expected 0 abortTransaction failed events, got %d", len(abortFailed))
 	})
 	t.Run("wrapped transient transaction error retried", func(t *testing.T) {
 		sess, err := client.StartSession()
@@ -397,15 +426,15 @@ func TestConvenientTransactions(t *testing.T) {
 
 		// returnError tracks whether or not the callback is being retried
 		returnError := true
-		res, err := sess.WithTransaction(context.Background(), func(sessCtx SessionContext) (interface{}, error) {
+		res, err := sess.WithTransaction(context.Background(), func(SessionContext) (interface{}, error) {
 			if returnError {
 				returnError = false
-				return nil, wrappedError{
+				return nil, fmt.Errorf("%w",
 					CommandError{
 						Name:   "test Error",
 						Labels: []string{driver.TransientTransactionError},
 					},
-				}
+				)
 			}
 			return false, nil
 		})
@@ -414,7 +443,7 @@ func TestConvenientTransactions(t *testing.T) {
 		assert.True(t, ok, "expected result type %T, got %T", false, res)
 		assert.False(t, resBool, "expected result false, got %v", resBool)
 	})
-	t.Run("expired context before commitTransaction does not retry", func(t *testing.T) {
+	t.Run("expired context before callback does not retry", func(t *testing.T) {
 		withTransactionTimeout = 2 * time.Second
 
 		coll := db.Collection("test")
@@ -430,12 +459,13 @@ func TestConvenientTransactions(t *testing.T) {
 		assert.Nil(t, err, "StartSession error: %v", err)
 		defer sess.EndSession(context.Background())
 
-		// Create context with short timeout.
-		withTransactionContext, cancel := context.WithTimeout(context.Background(), time.Nanosecond)
-		defer cancel()
-		callback := func() {
-			_, _ = sess.WithTransaction(withTransactionContext, func(sessCtx SessionContext) (interface{}, error) {
-				_, err := coll.InsertOne(sessCtx, bson.D{{}})
+		callback := func(ctx context.Context) {
+			// Create transaction context with short timeout.
+			withTransactionContext, cancel := context.WithTimeout(ctx, time.Nanosecond)
+			defer cancel()
+
+			_, _ = sess.WithTransaction(withTransactionContext, func(ctx SessionContext) (interface{}, error) {
+				_, err := coll.InsertOne(ctx, bson.D{{}})
 				return nil, err
 			})
 		}
@@ -443,7 +473,7 @@ func TestConvenientTransactions(t *testing.T) {
 		// Assert that transaction fails within 500ms and not 2 seconds.
 		assert.Soon(t, callback, 500*time.Millisecond)
 	})
-	t.Run("canceled context before commitTransaction does not retry", func(t *testing.T) {
+	t.Run("canceled context before callback does not retry", func(t *testing.T) {
 		withTransactionTimeout = 2 * time.Second
 
 		coll := db.Collection("test")
@@ -459,12 +489,13 @@ func TestConvenientTransactions(t *testing.T) {
 		assert.Nil(t, err, "StartSession error: %v", err)
 		defer sess.EndSession(context.Background())
 
-		// Create context and cancel it immediately.
-		withTransactionContext, cancel := context.WithTimeout(context.Background(), 2*time.Second)
-		cancel()
-		callback := func() {
-			_, _ = sess.WithTransaction(withTransactionContext, func(sessCtx SessionContext) (interface{}, error) {
-				_, err := coll.InsertOne(sessCtx, bson.D{{}})
+		callback := func(ctx context.Context) {
+			// Create transaction context and cancel it immediately.
+			withTransactionContext, cancel := context.WithTimeout(ctx, 2*time.Second)
+			cancel()
+
+			_, _ = sess.WithTransaction(withTransactionContext, func(ctx SessionContext) (interface{}, error) {
+				_, err := coll.InsertOne(ctx, bson.D{{}})
 				return nil, err
 			})
 		}
@@ -472,7 +503,7 @@ func TestConvenientTransactions(t *testing.T) {
 		// Assert that transaction fails within 500ms and not 2 seconds.
 		assert.Soon(t, callback, 500*time.Millisecond)
 	})
-	t.Run("slow operation before commitTransaction retries", func(t *testing.T) {
+	t.Run("slow operation in callback retries", func(t *testing.T) {
 		withTransactionTimeout = 2 * time.Second
 
 		coll := db.Collection("test")
@@ -509,11 +540,11 @@ func TestConvenientTransactions(t *testing.T) {
 		assert.Nil(t, err, "StartSession error: %v", err)
 		defer sess.EndSession(context.Background())
 
-		callback := func() {
-			_, err = sess.WithTransaction(context.Background(), func(sessCtx SessionContext) (interface{}, error) {
+		callback := func(ctx context.Context) {
+			_, err = sess.WithTransaction(ctx, func(ctx SessionContext) (interface{}, error) {
 				// Set a timeout of 300ms to cause a timeout on first insertOne
 				// and force a retry.
-				c, cancel := context.WithTimeout(sessCtx, 300*time.Millisecond)
+				c, cancel := context.WithTimeout(ctx, 300*time.Millisecond)
 				defer cancel()
 
 				_, err := coll.InsertOne(c, bson.D{{}})
@@ -528,7 +559,7 @@ func TestConvenientTransactions(t *testing.T) {
 }
 
 func setupConvenientTransactions(t *testing.T, extraClientOpts ...*options.ClientOptions) *Client {
-	cs := testutil.ConnString(t)
+	cs := integtest.ConnString(t)
 	poolMonitor := &event.PoolMonitor{
 		Event: func(evt *event.PoolEvent) {
 			switch evt.Type {
@@ -545,7 +576,7 @@ func setupConvenientTransactions(t *testing.T, extraClientOpts ...*options.Clien
 		SetReadPreference(readpref.Primary()).
 		SetWriteConcern(writeconcern.New(writeconcern.WMajority())).
 		SetPoolMonitor(poolMonitor)
-	testutil.AddTestServerAPIVersion(baseClientOpts)
+	integtest.AddTestServerAPIVersion(baseClientOpts)
 	fullClientOpts := []*options.ClientOptions{baseClientOpts}
 	fullClientOpts = append(fullClientOpts, extraClientOpts...)
 
@@ -555,7 +586,7 @@ func setupConvenientTransactions(t *testing.T, extraClientOpts ...*options.Clien
 	version, err := getServerVersion(client.Database("admin"))
 	assert.Nil(t, err, "getServerVersion error: %v", err)
 	topoKind := client.deployment.(*topology.Topology).Kind()
-	if compareVersions(t, version, "4.1") < 0 || topoKind == description.Single {
+	if compareVersions(version, "4.1") < 0 || topoKind == description.Single {
 		t.Skip("skipping standalones and versions < 4.1")
 	}
 
@@ -575,7 +606,7 @@ func getServerVersion(db *Database) (string, error) {
 	serverStatus, err := db.RunCommand(
 		context.Background(),
 		bson.D{{"serverStatus", 1}},
-	).DecodeBytes()
+	).Raw()
 	if err != nil {
 		return "", err
 	}
@@ -594,7 +625,7 @@ func getServerVersion(db *Database) (string, error) {
 //
 // Returns a positive int if version1 is greater than version2, a negative int if version1 is less
 // than version2, and 0 if version1 is equal to version2.
-func compareVersions(t *testing.T, v1 string, v2 string) int {
+func compareVersions(v1 string, v2 string) int {
 	n1 := strings.Split(v1, ".")
 	n2 := strings.Split(v2, ".")
 
